import { mapStateToControlProps, rankWith } from '@jsonforms/core';
import { html } from '@microsoft/fast-element';
import { GenesisType } from '../../utils';
import { isOneOfDate } from '../testers/isDate';
import { DATE_CONTROL_RANK } from './RenderersRanks';
/**
 * @internal
 */
const typeSwitch = (control) => {
    var _a, _b;
    if ((_a = control.schema) === null || _a === void 0 ? void 0 : _a.hasOwnProperty('oneOf')) {
        if (control.schema.oneOf.some((s) => s.genesisType === GenesisType.DATETIME)) {
            return 'datetime-local';
        }
        return 'date';
    }
    return ((_b = control.schema) === null || _b === void 0 ? void 0 : _b.genesisType) === GenesisType.DATETIME ? 'datetime-local' : 'date';
};
export const DateControlRendererTemplate = (prefix = 'zero') => html `
  <template>
    <control-wrapper
      :control=${(x) => x.control}
      :jsonForms=${(x) => x.jsonforms}
      :touched=${(x) => x.touched}
      ?submitted=${(x) => x.submitted}
      prefix="${prefix}"
    >
      <${prefix}-text-field
        :value=${(x) => {
    if (!x.control.data)
        return '';
    const type = typeSwitch(x.control);
    const date = new Date(x.control.data);
    const padded = (dateComponent) => dateComponent.toString().padStart(2, '0');
    const year = date.getUTCFullYear();
    const month = padded(date.getUTCMonth() + 1);
    const day = padded(date.getUTCDate());
    const hour = padded(date.getUTCHours());
    const minute = padded(date.getUTCMinutes());
    return (`${year}-${month}-${day}` + (type === 'datetime-local' ? ` ${hour}:${minute}` : ''));
}}
        id=${(x) => x.control.path}
        type="${(x) => typeSwitch(x.control)}"
        data-test-id=${(x) => x.control.path}
        ?required=${(x) => x.control.required}
        ?disabled=${(x) => !x.control.enabled}
        @blur=${(x, c) => {
    const dateValue = c.event.target.control.valueAsNumber;
    x.onBlur();
    x.control.handleChange(x.control.path, !isNaN(dateValue) ? dateValue : undefined);
}}"
        style="width: 100%"
      ></${prefix}-text-field>
    </control-wrapper>
  </template>
`;
export const DateControlRendererEntry = {
    renderer: html `
    ${(x) => DateControlRendererTemplate(x.prefix)}
  `,
    tester: rankWith(DATE_CONTROL_RANK, isOneOfDate),
    mapper: mapStateToControlProps, // TODO: change mapper to handle dates instead of doing it in the template
};
