import { ExpressionBuilder, Serialisers } from '@genesislcap/foundation-criteria';
const hour = 23;
const minute = 59;
const sec = 999;
/**
 * Creates a expressions/criteria from given payload
 * @internal
 */
export const createExpressions = (payload) => {
    const expressions = {
        options: [],
        dateOptions: [],
        numberOptions: [],
    };
    for (const [key, value] of Object.entries(payload)) {
        if ((value && !value.type) || (value && value.max) || (value && value.min)) {
            if (Array.isArray(value)) {
                const optionsExpressions = value.map((item) => {
                    return new ExpressionBuilder()
                        .withField(key)
                        .withValue(item)
                        .withSerialiser(Serialisers.EQ)
                        .build();
                });
                expressions.options = [...optionsExpressions, ...expressions.options];
            }
            else if (value && (value.min || value.max)) {
                if (value.type == 'number') {
                    expressions.numberOptions = [
                        ...dateNumberExpressions(key, value, 'number'),
                        ...expressions.numberOptions,
                    ];
                }
                if (value.type == 'date') {
                    expressions.dateOptions = [
                        ...dateNumberExpressions(key, value, 'date'),
                        ...expressions.dateOptions,
                    ];
                }
            }
            else {
                const stringOptions = new ExpressionBuilder()
                    .withField(key)
                    .withValue(value)
                    .withSerialiser(Serialisers.EQ)
                    .build();
                expressions.options = [stringOptions, ...expressions.options];
            }
        }
    }
    return expressions;
};
const dateNumberExpressions = (key, value, type) => {
    let startDay;
    let endDay;
    if (type == 'date') {
        startDay = value.min ? convertTimestampToDate(value.min) : undefined;
        endDay = value.max ? convertTimestampToDate(value.max) : undefined;
    }
    const expressions = [];
    const startExpression = value.min
        ? new ExpressionBuilder()
            .withField(key)
            .withValue(startDay !== null && startDay !== void 0 ? startDay : value.min)
            .withSerialiser(startDay ? Serialisers.dateIsGreaterEqual : Serialisers.GE)
            .build()
        : undefined;
    const endExpression = value.max
        ? new ExpressionBuilder()
            .withField(key)
            .withValue(endDay !== null && endDay !== void 0 ? endDay : value.max)
            .withSerialiser(endDay ? Serialisers.dateIsLessEqual : Serialisers.LE)
            .build()
        : undefined;
    startExpression ? expressions.push(startExpression) : '';
    endExpression ? expressions.push(endExpression) : '';
    return expressions;
};
const convertTimestampToDate = (timestamp) => {
    const months = (new Date(timestamp).getUTCMonth() + 1).toString().padStart(2, '0');
    const days = new Date(timestamp).getUTCDate().toString().padStart(2, '0');
    const year = new Date(timestamp).getUTCFullYear().toString();
    return [year, months, days].join('');
};
